function h=antplot(varargin)
% antplot.m :
% read antenna parameters generated by antest.m and plot them. antenna
% parameters file is specified in options or with menu 'Plot'-'Read Data...'.
% plot content is selected in options or with menu 'Plot'-'PCV Skyplot' etc.
% to plot antenna model of antex format, select antex file with menu
% 'Plot'-'Read Model', and execute menu 'Plot'-'PCV Skyplot (Model)' etc.
% to zoom or move axis in graph, use mouse drag at x or y axis of graph.
% 
% options : ([] default)
%    file             antenna parameter file generated by antest.m
%    '-pcvs'          plot pcv skyplot [default]
%    '-pcve'          plot pcv contour
%    '-pmpt'          plot carrier-phase multipath-time
%    '-pmpe'          plot carrier-phase multipath-elevation
%    '-pmps'          plot carrier-phase multipath skyplot
%    '-pmpt'          plot code multipath-time
%    '-cmpe'          plot code multipath-elevation
%    '-cmps'          plot code multipath skyplot
%    '-snre'          plot C/N0-elevation
%    '-snrs'          plot C/N0 skyplot
%    '-ant',ant       antenna/receiver name in title ['']
%    '-out',file      output jpeg image to file and close [no output]
%    '-posu',[x;y;z]  antenna reference position (ecef) (m) [same as analysis]
%    '-filt',filt     filter window of multipath (s) [100]
%    '-range',range   plot range (code:m,phase:cm) [3]
%    '-view',azel     view angle of 3d skyplot [45 30]
%    '-tlim',lim      time range [tmin,tmax] (hr) [all range]
%    '-elev'          show elevation angle in multipath-time plot [hide]
%    '-label'         show contour label in pcv contour [hide]
%    '-noave'         hide averate in multipath-elevation plot [show]
%    '-nopco'         hide phase center offset [show]
%    '-ltype',type    line style (1:line,2:dot,3:line and dot) [1]
%
%            Copyright(c) 2007-2011 by T.Takasu, all rights reserved
%
% version: $Revision: 1.2 $ $Date: 2008/06/04 03:31:42 $
% history: 2008/05/23 1.0 new
%          2009/01/14 1.1 fix bug on error stop if no analysis data
%          2011/09/13 1.2 support GLONASS, QZSS and L5

if nargin>0&strncmp(varargin{1},'cb_',3), feval(varargin{:}); return; end

p.style='pcvs';
p.ant=''; 
p.mant=''; 
p.file='';
p.outf='';
p.antex=0;
p.posu=[];
p.opt.filt=100;
p.opt.sat=1:32+24+3;
p.opt.range=3; 
p.opt.view=[45,30]; 
p.opt.tlim=[];
p.opt.showf=[0 0 1 1];
p.opt.ltype=1;
p.opt.freq=1;
p.opt.font={'Times New Roman',9};
p.opt.color={'b','r'};
if ~isempty(p.outf), p.opt=loadopt(p.opt); end
p.t0=[]; p.pco=[]; p.pcv=[]; p.pmp=[]; p.cmp=[]; p.snr=[]; p.nmax=[]; p.ru=[];
p.rr=[]; p.mants={}; p.mpco=[]; p.mpce=[]; p.mpcv=[]; p.mels=[]; p.mazs=[];
i=1;
while i<=nargin
    arg=varargin{i};
    switch arg
    case '-ant',   i=i+1; p.ant=varargin{i};
    case '-out',   i=i+1; p.outf=varargin{i};
    case '-posu',  i=i+1; p.posu=opttonum(varargin{i});
    case '-filt',  i=i+1; p.opt.filt=opttonum(varargin{i});
    case '-range', i=i+1; p.opt.range=opttonum(varargin{i});
    case '-view',  i=i+1; p.opt.view=opttonum(varargin{i});
    case '-tlim',  i=i+1; p.opt.tlim=opttonum(varargin{i});
    case '-elev',  p.opt.showf(1)=1;
    case '-label', p.opt.showf(2)=1;
    case '-noave', p.opt.showf(3)=0;
    case '-nopco', p.opt.showf(4)=0;
    case '-ltype', i=i+1; p.opt.ltype=opttonum(varargin{i});
    otherwise, if arg(1)=='-', p.style=arg(2:end); else p.file=arg; end
    end
    i=i+1;
end
% generate window/menu
h=uifunc('newfig','ANTPLOT','renderer','painters');
menu={'Read &Data...','Read &Model...','-','&PCV Skyplot','PCV &Contour',...
      'Phase Multipath-&Time','Phase Multipath-&Elevation','Phase Multipath &Skyplot',...
      'Code Multipath-Time','Code Multipath-Elevation','Code Multipath Skyplot',...
      '&C/N0-Elevation','C/N0 Skyplot','-','PCV Skyplot (Model)','PCV Contour (Model)',...
      '-','&Options...'};
uifunc('newmenu','plot','&Plot',menu,[mfilename,' cb_plot'],4);
guidata(h,p);
if isempty(p.file), return; end

% read data and plot
readdata;

% save plot file
if ~isempty(p.outf), outplot(gcf,p.outf); close(gcf); end

% option to number -------------------------------------------------------------
function n=opttonum(s), if isstr(s), n=str2num(s); else n=s; end

% callback menu plot -----------------------------------------------------------
function cb_plot
p=guidata(gcf); 
switch get(gcbo,'userdata');
case 1 % read file
    filt={'*.*','All Files (*.*)';'*.mat','Matlab Files (*.mat)'};
    file=uifunc('readdlg',filt,'Read Data');
    if isempty(file), return; else p.file=file{1}; end
    guidata(gcf,p);
    readdata;
    return;
case 2 % read model
    filt={'*.atx','Antex Files (*.atx)';'*.*','All Files (*.*)'};
    file=uifunc('readdlg',filt,'Read Model');
    if isempty(file), return; else p.file=file{1}; end
    guidata(gcf,p);
    readmodel;
    return;
case 3,  p.style='pcvs';
case 4,  p.style='pcve';
case 5,  p.style='pmpt';
case 6,  p.style='pmpe';
case 7,  p.style='pmps';
case 8,  p.style='cmpt';
case 9,  p.style='cmpe';
case 10, p.style='cmps';
case 11, p.style='snre';
case 12, p.style='snrs';
case 13, p.style='pcvsm';
case 14, p.style='pcvem';
case 15, optdlg; return;
end
uifunc('chkmenu',gcbo); guidata(gcf,p); updateplot;

% callback menu sat ------------------------------------------------------------
function cb_sat
p=guidata(gcf);
label=get(gcbo,'label');
if     label=='ALL', p.opt.sat=1:32+24+3;
elseif label=='GPS', p.opt.sat=1:32;
elseif label=='GLO', p.opt.sat=32+(1:24);
elseif label=='QZS', p.opt.sat=56+(1:3);
else p.opt.sat=str2sat(label);
end
uifunc('chkmenu',gcbo); guidata(gcf,p); updateplot;

% callback menu ant ------------------------------------------------------------
function cb_ant
p=guidata(gcf);
p.mant=get(gcbo,'label');
uifunc('chkmenu',gcbo); guidata(gcf,p); updateplot;

% options dialog ---------------------------------------------------------------
function optdlg
p=guidata(gcf);
font.FontName=p.opt.font{1};
font.FontSize=p.opt.font{2};
font.FontUnits='points';
font.FontWeight='normal';
font.FontAngle='normal';
uifunc('newdlg','Options',[250,319]);
uifunc('newtxts',[15,305,150,23],{'MP Filter Window (s)','Axis Range (cm/m)',...
       '3D View Angles (deg)','Show Elevation','Show Contour Label','Show Average',...
       'Show Offset','Model Frequency','Data Color',...
       'Average/Elev Color','Plot Style','Font'});
h1=uifunc('newedt',[150,288,90,20],num2str(p.opt.filt));
h2=uifunc('newedt',[150,265,90,20],num2str(p.opt.range));
h3=uifunc('newedt',[150,242,45,20],num2str(p.opt.view(1)));
h4=uifunc('newedt',[195,242,45,20],num2str(p.opt.view(2)));
h5=uifunc('newpop',[150,219,92,20],{'OFF','ON'},p.opt.showf(1)+1);
h6=uifunc('newpop',[150,196,92,20],{'OFF','ON'},p.opt.showf(2)+1);
h7=uifunc('newpop',[150,173,92,20],{'OFF','ON'},p.opt.showf(3)+1);
h8=uifunc('newpop',[150,150,92,20],{'OFF','ON'},p.opt.showf(4)+1);
h9=uifunc('newpop',[150,127,92,20],{'L1','L2','L5'},p.opt.freq);
h10=uifunc('newcolorsel',[150,104,90,20],p.opt.color{1});
h11=uifunc('newcolorsel',[150,81,90,20],p.opt.color{2});
h12=uifunc('newpop',[150,58,92,20],{'Line','Dot','Line/Dot'},p.opt.ltype);
h13=uifunc('newfontsel',[40,35,200,20],font);
uifunc('newokcancel',[81,6,160,23]);
if ~uifunc('waitok'), close; return; end
p.opt.filt=str2num(get(h1,'string')); if isempty(p.opt.filt), p.opt.filt=0; end
p.opt.range=str2num(get(h2,'string')); if isempty(p.opt.range), p.opt.range=3; end
p.opt.view=[str2num(get(h3,'string')),str2num(get(h4,'string'))];
if length(p.opt.view)<2, p.opt.view=[45 30]; end
p.opt.showf=[get(h5,'value'),get(h6,'value'),get(h7,'value'),get(h8,'value')]-1;
p.opt.freq=get(h9,'value');
p.opt.color{1}=get(h10,'backgroundcolor');
p.opt.color{2}=get(h11,'backgroundcolor');
p.opt.ltype=get(h12,'value');
font=get(h13,'userdata');
p.opt.font{1}=font.FontName;
p.opt.font{2}=font.FontSize;
close
saveopt(p.opt);
guidata(gcf,p); updateplot;

% read data --------------------------------------------------------------------
function readdata(file)
p=guidata(gcf);
t0=[]; pco=[]; pcv=[]; pmp=[]; cmp=[]; ru=[]; rr=[]; opt=[];
try
    load('-mat',p.file);
catch
    disp(['file read error : ',p.file]); return;
end
p.t0=t0; p.pco=pco; p.pcv=pcv; p.pmp=pmp; p.cmp=cmp; p.snr=snr; p.nmax=opt.nmax;
p.ru=ru; p.rr=rr;
set(gcf,'name',['ANTPLOT : ',p.file]);
delete(findobj(gcf,'tag','sat')); 
menu={'ALL','GPS','GLO','QZS','-'};
if ~isempty(pmp)
    for i=unique(pmp(:,2))', menu={menu{:},sat2str(i)}; end
elseif ~isempty(cmp)
    for i=unique(cmp(:,2))', menu={menu{:},sat2str(i)}; end
end
uifunc('newmenu','sat','&Sat',menu,[mfilename,' cb_sat'],1);
p.opt.sat=1:32+24+3;
guidata(gcf,p);
updateplot;

% read model -------------------------------------------------------------------
function readmodel(file)
p=guidata(gcf);
[p.mants,p.mpco,p.mpce,p.mpcv,p.mels,p.mazs]=readatx(p.file);
if isempty(p.mants), disp(['no antenna model : ',p.file]); return; end
p.mant=p.mants{1};
guidata(gcf,p);
delete(findobj(gcf,'tag','ant')); 
uifunc('newmenu','ant','&Ant',p.mants,[mfilename,' cb_ant'],1);
updateplot;

% load options -----------------------------------------------------------------
function opt=loadopt(opt)
[path,file,ext]=fileparts(which(mfilename));
file=fullfile(path,[file,'.opt']);
if exist(file), load(file,'-mat','opt'); end

% save options -----------------------------------------------------------------
function saveopt(opt)
[path,file,ext]=fileparts(which(mfilename));
save(fullfile(path,[file,'.opt']),'-mat','opt');

% update plot ------------------------------------------------------------------
function updateplot
p=guidata(gcf);
switch p.style
case {'pcvs','pcve'}, plotpcv;
case {'pcvsm','pcvem'}, plotpcvm;
case 'pmpt', plotmpt(p.t0,p.pmp,p.opt,'cm','Carrier Phase',p.ant);
case 'pmps', plotmps(p.pmp,p.opt,'cm','Carrier Phase',p.ant);
case 'pmpe', plotmpe(p.pmp,p.opt,'cm','Carrier Phase',p.ant);
case 'cmpt', plotmpt(p.t0,p.cmp,p.opt,'m','Code',p.ant);
case 'cmps', plotmps(p.cmp,p.opt,'m','Code',p.ant);
case 'cmpe', plotmpe(p.cmp,p.opt,'m','Code',p.ant);
case 'snrs', plotsnrs(p.snr,p.opt,p.ant);
case 'snre', plotsnre(p.snr,p.opt,p.ant);
otherwise, disp(['style error : ',p.style]); return;
end

% plot pcv ---------------------------------------------------------------------
function plotpcv
p=guidata(gcf);
if isempty(p.pcv), clf; return; end
if strcmp(p.style,'pcvs'), els=5:5:90; azs=0:5:360; else els=0:2:90; azs=0:2:360; end
for i=1:length(azs)
    for j=1:length(els)
        pca(i,j)=p.pcv'*sphfun([azs(i),els(j)]*pi/180,p.nmax)';
    end
end
if length(p.posu)==3
    [pos,E]=togeod(p.ru); posc=E'*p.pco+p.ru;
    [pos,E]=togeod(p.posu); p.pco=E*(posc-p.posu);
end
switch p.style
case 'pcvs', plotpcvs(p.pco,pca,azs,els,p.opt,'cm',p.ant);
case 'pcve', plotpcve(p.pco,pca,azs,els,p.opt,'cm',p.ant);
end

% plot pcv model----------------------------------------------------------------
function plotpcvm
flabel={'L1','L2','L5'};
p=guidata(gcf);
i=find(strcmp(p.mants,p.mant));
if isempty(i), clf; return; else i=i(1); end
pco=p.mpco{p.opt.freq}(i,:); pce=p.mpce{p.opt.freq}(i,:);
pcv=p.mpcv{p.opt.freq}(:,:,i);
if isnan(pcv(1,1)), pcv=repmat(pce,length(p.mazs),1); end
ant=sprintf('%s (%s)',p.mants{i},flabel{p.opt.freq});
switch p.style
case 'pcvsm', plotpcvs(pco,pcv,p.mazs,p.mels,p.opt,'cm',ant);
case 'pcvem', plotpcve(pco,pcv,p.mazs,p.mels,p.opt,'cm',ant);
end

% plot antenna pcv in skyplot --------------------------------------------------
function plotpcvs(pco,pca,azs,els,opt,unit,ant)
clf; font={'fontname',opt.font{1},'fontsize',opt.font{2}}; cl=[.7 .7 .7]; t='';
if strcmp(unit,'cm'), pco=pco*100; pca=pca*100; end
axes('xlim',[-0.88 0.88],'ylim',[-0.88 0.88],'zlim',[-opt.range opt.range],...
     'clim',[-opt.range opt.range],'plotboxaspectratio',[1,1,0.2]);
axis off; hold on; 
[az,el]=meshgrid(azs,els);
[x,y]=spos(az,el);
surf(x,y,pca','edgecolor',cl,'facecolor','interp','facelighting','phong');
plotsky(opt.font,[.5 .5 .5]);
view(opt.view(1),opt.view(2)); camproj('perspective'); plotzs(opt.range,font{:});
if length(pco)>=3&opt.showf(4)
    s=sprintf('Offset: E=%6.2f  N=%6.2f U=%6.2f %s',pco,unit); t='Offset/';
    text(0.5,0.1,s,'units','normalized','horizontal','center',font{:});
end
text(0.85,0.1,['(',unit,')'],'units','normalized','horizontal','center',font{:});
set(colorbar('horiz'),'position',[0.15 0.08 0.7 0.02],font{:});
title(['Antenna Phase Center ',t,'Variation : ',ant],'interpreter','none',font{:});
set(gca,'position',[0 0.05 1 0.95]);

% plot antenna pcv contour plot ------------------------------------------------
function plotpcve(pco,pca,azs,els,opt,unit,ant)
clf; font={'fontname',opt.font{1},'fontsize',opt.font{2}}; cl=[.7 .7 .7]; t='';
if strcmp(unit,'cm'), pco=pco*100; pca=pca*100; end
axes(font{:},'clim',[-opt.range opt.range],'color','none'); hold on;
xt=-opt.range:0.1:opt.range;
[az,el]=meshgrid(azs,els);
[c,hh]=contourf(az,el,pca',xt);
set(hh,'edgecolor',cl);
if opt.showf(2), set(clabel(c,hh),font{:}); end
xlim([0 360]); ylim([0 90]);
set(gca,'xtick',0:30:360,'ytick',0:15:90);
xlabel('Azimuth Angle (\circ)');
ylabel('Elevation Angle (\circ)');
for az=30:30:330, plot([az,az],[0,90],':','color','k'); end
for el=15:15:75, plot([0,360],[el,el],':','color','k'); end
plot([0,0,360,360,0],[0,90,90,0,0],'-','color','k');
if length(pco)>=3&opt.showf(4)
    s=sprintf('Offset: E=%6.2f  N=%6.2f  U=%6.2f %s',pco,unit); t='Offset/';
    text(0.98,0.96,s,'units','normalized','horizontal','right',font{:});
end
text(1,-0.09,['(',unit,')'],'units','normalized','horizontal','right',font{:});
xl={};
for i=1:length(xt)
    if mod(xt(i),1), s=''; else s=num2str(xt(i)); end, xl={xl{:},s};
end
set(colorbar('horiz'),'xtick',xt,'xticklabel',xl,...
    'position',[0.11 0.08 0.78 0.02],font{:});
title(['Antenna Phase Center ',t,'Variation : ',ant],'interpreter','none',font{:});
set(gca,'position',[0.09 0.2 0.82 0.68]);

% plot multipath in time graph -------------------------------------------------
function plotmpt(t0,mp,opt,unit,ti,ant)
clf; font={'fontname',opt.font{1},'fontsize',opt.font{2}};
if isempty(mp), return; end
if strcmp(unit,'cm'), mp(:,5)=mp(:,5)*100; end
if opt.filt>0, mp=avefilt(mp,opt.filt); end
el=0:5:90;
axes('position',[0.1,0.1,0.8,0.8],font{:});
hold on; grid on; box on; s=pstyle(opt);
rms=mprms(mp,opt.sat);
i=[];
for j=1:length(opt.sat)
    k=find(mp(:,2)==opt.sat(j)); i=[i;k];
    mp(k(diff(mp(k,1))>30),5)=nan;
    plot(mp(k,1)/3600,mp(k,5),s{:});
end
if isempty(opt.tlim), xl=[min(mp(:,1)) max(mp(:,1))]/3600; else xl=opt.tlim; end
xlim(xl); ylim([-opt.range,opt.range]);
if ~isempty(t0), dv=datevec(t0); s=sprintf(' : %04d/%02d/%02d',dv(1:3)); else s=''; end
xlabel(['Time (hr)',s]); ylabel(['Multipath (',unit,')']);
title([ti,' Multipath : ',ant],'interpreter','none',font{:});
s=sprintf('RMS=%.2f %s',rms,unit);
text(0.98,0.98,s,'units','normalized','horizontal','right','vertical','top',font{:});
uifunc('moveax',gca,[1 0 1 1]);
if ~opt.showf(1), return; end
axes('position',get(gca,'position'),'color','none','yaxislocation','right',...
     'ycolor',opt.color{2},'xtick',[],'ticklength',[0 0.01],font{:});
hold on; grid off;
xlim(xl); ylim([0,90]); ylabel('Elevation Angle (\circ)');
for i=1:length(opt.sat)
    j=find(mp(:,2)==opt.sat(i));
    mp(j(diff(mp(j,1))>30),4)=nan;
    plot(mp(j,1)/3600,mp(j,4)*180/pi,'-','color',opt.color{2});
end
uifunc('moveax',gca,[1 0 1 0]);

% plot multipath in elevation graph --------------------------------------------
function plotmpe(mp,opt,unit,ti,ant)
clf; font={'fontname',opt.font{1},'fontsize',opt.font{2}};
if isempty(mp), return; end
if strcmp(unit,'cm'), mp(:,5)=mp(:,5)*100; end
if opt.filt>0, mp=avefilt(mp,opt.filt); end
el=0:5:90;
axes('position',[0.1,0.1,0.85,0.8],font{:});
hold on; grid on; box on; s=pstyle(opt);
rms=mprms(mp,opt.sat);
i=[];
for j=1:length(opt.sat)
    k=find(mp(:,2)==opt.sat(j)); i=[i;k];
    mp(k(diff(mp(k,1))>30),5)=nan;
    plot(mp(k,4)*180/pi,mp(k,5),s{:});
end
if opt.showf(3)
    avemp=repmat(nan,length(el)); stdmp=avemp;
    for j=1:length(el)
        k=i(el(j)-2.5<=mp(i,4)*180/pi&mp(i,4)*180/pi<el(j)+2.5&~isnan(mp(i,5)));
        if ~isempty(k), avemp(j)=mean(mp(k,5)); stdmp(j)=std(mp(k,5)); end
    end
    plot(el,avemp,'.-','color',opt.color{2});
end
xlabel('Elevation Angle (\circ)');
ylabel(['Multipath (',unit,')']);
xlim([0 90]); ylim([-opt.range,opt.range]);
set(gca,'xtick',0:15:90);
title([ti,' Multipath : ',ant],'interpreter','none',font{:});
uifunc('moveax',gca,[0 0 0 1]);
s=sprintf('RMS=%.2f %s',rms,unit);
text(0.98,0.98,s,'units','normalized','horizontal','right','vertical','top',font{:});

% plot multipath in skyplot ----------------------------------------------------
function plotmps(mp,opt,unit,ti,ant)
clf; font={'fontname',opt.font{1},'fontsize',opt.font{2}};
if isempty(mp), return; end
if strcmp(unit,'cm'), mp(:,5)=mp(:,5)*100; end
if opt.filt>0, mp=avefilt(mp,opt.filt); end
axes('xlim',[-0.88 0.88],'ylim',[-0.88 0.88],'zlim',[-opt.range opt.range],...
     'clim',[-opt.range opt.range],'plotboxaspectratio',[1,1,0.2]);
axis off; hold on; s=pstyle(opt);
rms=mprms(mp,opt.sat);
for i=1:length(opt.sat)
    j=find(mp(:,2)==opt.sat(i));
    az=mp(j,3)*180/pi; el=mp(j,4)*180/pi;
    mp(j(find(diff(az).^2+diff(el).^2>9)+1),5)=nan;
    splot3(az,el,mp(j,5),s{:});
end
plotsky(opt.font,[.5 .5 .5]);
view(opt.view(1),opt.view(2)); camproj('perspective'); plotzs(opt.range,font{:});
text(0.8,0.1,['(',unit,')'],'units','normalized','horizontal','center',font{:});
set(gca,'position',[0 0 1 1]);
title([ti,' Multipath : ',ant],'interpreter','none',font{:});
s=sprintf('RMS=%.2f %s',rms,unit);
text(0.2,0.1,s,'units','normalized','horizontal','left',font{:});

% plot snr in elevation graph -------------------------------------------------
function plotsnre(snr,opt,ant)
clf; font={'fontname',opt.font{1},'fontsize',opt.font{2}};
if isempty(snr), return; end
el=0:5:90; range=[20 60];
axes('position',[0.1,0.1,0.85,0.8],font{:});
hold on; grid on; box on;
switch opt.ltype, case 1, s='-'; case 2, s='.'; case 3, s='.-'; end 
k=[];
for i=1:length(opt.sat)
    j=find(snr(:,2)==opt.sat(i)); k=[k;j];
    plot(snr(j,4)*180/pi,snr(j,5),s,'color',opt.color{1});
end
avesn=repmat(nan,length(el)); stdsn=avesn;
for i=1:length(el)
    j=find(el(i)-2.5<=snr(k,4)*180/pi&snr(k,4)*180/pi<el(i)+2.5&~isnan(snr(k,5)));
    if ~isempty(j), avesn(i)=mean(snr(k(j),5)); stdmp(i)=std(snr(k(j),5)); end
end
if opt.showf(3)
    plot(el,avesn,'.-','color',opt.color{2});
end
xlabel('Elevation Angle (\circ)'); ylabel('C/N0 (dBHz)');
xlim([0 90]); ylim(range);
set(gca,'xtick',0:15:90);
title(['Carrier Power To Noise Density (C/N0) : ',ant],'interpreter','none',font{:});

% plot snr in skyplot ----------------------------------------------------------
function plotsnrs(snr,opt,ant)
clf; font={'fontname',opt.font{1},'fontsize',opt.font{2}};
if isempty(snr), return; end
range=[20 60]; rs=2.5;
axes('xlim',[-1.2 1.2],'ylim',[-1.2 1.2]);
axis equal; axis off; hold on; 
set(gca,'clim',range);
step=range(1):rs:range(2); step([1,end])=[-inf,inf];
cs=get(gcf,'colormap');
for i=1:length(opt.sat)
    j=find(snr(:,2)==opt.sat(i)); az=snr(j,3)*180/pi; el=snr(j,4)*180/pi;
    for k=1:length(step)
        n=find(step(k)-rs/2<=snr(j,5)&snr(j,5)<step(k)+rs/2);
        color=cs(floor(k/length(step)*(size(cs,1))-1)+1,:);
        splot(az(n),el(n),'.','markersize',5,'color',color);
    end
end
plotsky(opt.font,[.5 .5 .5]);
set(gca,'position',[0 0 1 0.95]);
uifunc('moveax');
text(0.85,0.1,'(dBHz)','units','normalized','horizontal','right',font{:});
title(['Carrier Power To Noise Density (C/N0) : ',ant],'interpreter','none',font{:});
axes('position',[0.9 0.1 0.015 0.8],'xlim',[0 1],'ylim',range,'xtick',[],...
     'yaxislocation','right','ticklength',[0 0],font{:});
hold on; box on;
step=range(1):rs:range(2);
for i=1:length(step)
    patch([0 1 1 0],step(i)+[-rs -rs rs rs]/2,0,'edgecolor',[.5 .5 .5],...
          'facecolor',cs(floor(i/length(step)*(size(cs,1))-1)+1,:));
end

% draw skyplot -----------------------------------------------------------------
function h=plotsky(font,color)
h=[]; label='NESWN'; font={'fontname',font{1},'fontsize',font{2}};
for az=0:30:330
    if mod(az,90)==0, a=-5; t='-'; s=label(az/90+1); else a=0; t=':'; s=num2str(az); end
    h=[h;splot([az,az],[a,90],t,'color',color)];
    h=[h;stext(az,-10,s,'horizontal','center',font{:})];
end
for el=0:15:75
    if el==0, t='-'; else t=':'; end
    h=[h;splot(0:5:360,el*ones(1,73),t,'color',color)];
    if el>0, h=[h;stext(0,el,[' ',num2str(el)],font{:})]; end
end

% plot in skyplot --------------------------------------------------------------
function h=splot(az,el,varargin)
[x,y]=spos(az,el); h=plot(x,y,varargin{:});

% plot 3d in skyplot -----------------------------------------------------------
function h=splot3(az,el,z,varargin)
[x,y]=spos(az,el); h=plot3(x,y,z,varargin{:});

% text in skyplot --------------------------------------------------------------
function h=stext(az,el,varargin)
[x,y]=spos(az,el); h=text(x,y,varargin{:});

% position in skyplot ----------------------------------------------------------
function [x,y]=spos(az,el)
x=(90-el)/90.*sin(az/180*pi);
y=(90-el)/90.*cos(az/180*pi);

% plot zscale in skyplot -------------------------------------------------------
function plotzs(range,varargin)
r=1.3;
for i=[-1 1]
    plot3([0 0],i*[1.3 1.3],[-range range],'k-');
    for j=-1:1
        plot3([0 0],i*(r+[-0.03 0.03]),j*[range range],'k-');
        text(0,i*(r+0.12),j*range,num2str(j*range),'horizontal','center',varargin{:});
    end
end

% running average filter -------------------------------------------------------
function mp=avefilt(mp,filt)
if filt<=0, return; end
for sat=1:32
    i=find(mp(:,2)==sat);
    mpa=repmat(nan,length(i),1);
    for j=1:length(i)
        t=mp(i(j),1);
        for n=j:-1:1,      if mp(i(n),1)<=t-filt/2, break; end, end
        for m=j:length(i), if mp(i(m),1)>=t+filt/2, break; end, end
        k=i(n:m); k=k(~isnan(mp(k,5)));
        if ~isempty(k), mpa(j)=sum(mp(k,5))/length(k); end
    end
    mp(i,5)=mpa;
end

% rms of multipath -------------------------------------------------------------
function rms=mprms(mp,sat)
i=[]; for s=sat, i=[i;find(mp(:,2)==s)]; end
i=i(~isnan(mp(i,5)));
if ~isempty(i), rms=sqrt(mp(i,5)'*mp(i,5)/length(i)); else rms=0; end

% plot style -------------------------------------------------------------------
function s=pstyle(opt)
switch opt.ltype
case 2, s={'.','color',opt.color{1},'markersize',3};
case 3, s={'.-','color',opt.color{1}};
otherwise, s={'-','color',opt.color{1}};
end 

% output plot file -------------------------------------------------------------
function outplot(h,file)
if ~isempty(h), imwrite(frame2im(getframe(h)),[file,'.jpg'],'jpeg'); end

% date string ------------------------------------------------------------------
function s=dstr(t0), dv=datevec(t0); s=sprintf('%04d/%02d/%02d',dv(1:3));

% sat to string ----------------------------------------------------------------
function s=sat2str(sat)
l='G'; if sat>56, l='J'; sat=sat-56; elseif sat>32, l='R'; sat=sat-32; end
s=sprintf('%s%02d',l,sat);

% string to sat ----------------------------------------------------------------
function sat=str2sat(s)
sat=sscanf(s,'%*c%d');
if s(1)=='R', sat=sat+32; elseif s(1)=='J', sat=sat+56; end
