function [cmp,snr]=antcode(t0,obs,iobs,nav,inav,ru,opt)
% antcode.m :
% estimate antenna parameters with code observations.
%
%            Copyright(c) 2007-2011 by T.Takasu, all rights reserved
%
% version: $Revision: 1.3 $ $Date: 2008/07/21 06:45:05 $
% history: 2008/05/23 0.1 new
%          2008/07/21 0.2 fix bug on checking LLI for slip detection
%          2009/01/07 0.3 fix bug on error-stop in case of duplicated obs data
%          2009/01/14 0.4 fix bug on deleting duplicated obs data
%          2011/09/13 0.5 support GLONASS, QZSS and L5

nsat=32+24+3;
f0=[1.57542E9,1.22760E9,1.17645E9];
f1=[1.60200E9,1.24600E9]; f2=[0.56250E6,0.43750E6];
f=repmat(f0,nsat,1);
for sat=32+(1:24)
    i=find(inav==sat);
    if isempty(i), f(sat,:)=nan; else f(sat,:)=[f1+f2*nav(i(1),17),nan]; end
end
% ionospheric delay variation with reference antenna
i=find(iobs(:,3)==2);
iono=ionovar(obs(i,:),iobs(i,:),f);

% code multipath and snr with target antenna/receiver
i=find(iobs(:,3)==1);
obs=obs(i,:); iobs=iobs(i,:);

cmp=[]; snr=[]; mp=[];
for sat=1:nsat
    i=find(iobs(:,2)==sat);
    j=find(diff(iobs(i,1))>60); % arc break
    for k=[1 j'+1;j' length(i)]
        n=i(k(1):k(2)); if isempty(n), continue; end
        disp(sprintf('TIME=%6.0f-%6.0f : %s N=%5d',iobs(i(k(1:2)),1),sat2str(sat),length(n)));
        mp=[mp;multip(obs(n,:),iobs(n,:),nav,inav,iono{sat},opt,ru,f(sat,:))];
    end
end
if ~isempty(mp), cmp=mp(:,1:5); snr=mp(:,[1:4,6]); end

% ionospheric delay variations -------------------------------------------------
function iono=ionovar(obs,iobs,f)
nsat=32+24+3;
for sat=1:nsat
    iono{sat}={};
    i=find(iobs(:,2)==sat);
    j=find(diff(iobs(i,1))>60); % arc break
    for k=[1 j'+1;j' length(i)]
        n=i(k(1):k(2));
        ions=ionov(obs(n,:),iobs(n,:),f(sat,:));
        iono{sat}={iono{sat}{:},ions{:}};
    end
end

% ionospheric delay variation of each arc --------------------------------------
function ions=ionov(obs,iobs,f)
ions={};
ion=obs(:,1:2)*[-1;1]/(1-f(1)^2/f(2)^2); % L1 iono (m)
if size(iobs,1)>=2, tint=iobs(2,1)-iobs(1,1); else return; end
i=find(~isnan(ion));
i=i(abs(diff(ion(i)))>0.01*tint);
i=unique([i;find(bitand(iobs(:,4),3))-1]); % cycle slip
for j=[1 i'+1;i' length(ion)]
    t=iobs(j(1):j(2),1); 
    ione=ion(j(1):j(2));
    k=find(~isnan(ione)); if length(k)<2, continue; end
    ione=ione-mean(ione(k));
    ione=interp1(t(k),ione(k),t);
    ions={ions{:},[t,ione]};
end

% code multipath and snr -------------------------------------------------------
function mp=multip(obs,iobs,nav,inav,iono,opt,ru,f)
fact=1;
if opt.f==2, fact=f(1)^2/f(2)^2; elseif opt.f==3, fact=f(1)^2/f(3)^2; end
mp=[]; [pos,E]=togeod(ru);
gf=obs(:,opt.f+3)-obs(:,opt.f); % code-phase
i=find(~isnan(gf));
i=i(abs(diff(gf(i)))>3);
i=unique([i;find(bitand(iobs(:,4),3))-1]); % cycle slip
for j=[1 i'+1;i' length(gf)]
    k=j(1):j(2);
    t=iobs(k,1); mp0=gf(k); obsk=obs(k,:); iobsk=iobs(k,:);
    for k=1:length(iono)
        [tt,n,m]=intersect(round(t),round(iono{k}(:,1)));
        if length(n)<2, continue; end
        mp1=mp0(n)-2*fact*iono{k}(m,2); if all(isnan(mp1)), continue; end
        mp1=mp1-mean(mp1(~isnan(mp1)));
        azel=sazel(obsk(n,:),iobsk(n,:),nav,inav,ru,E);
        mp=[mp;iobsk(n,1:2),azel,mp1,obsk(n,opt.f+6)];
    end
end

% satellite azimuth/elevation angle --------------------------------------------
function azel=sazel(obs,iobs,nav,inav,ru,E)
azel=repmat(nan,size(obs,1),2);
for i=1:size(obs,1)
    rs=ephpos(obs(i,:),iobs(i,:),nav,inav);
    [r,e]=georng(rs,ru);
    a=E*e;
    azel(i,:)=[atan2(a(1),a(2)),asin(a(3))];
end

% sat to string ----------------------------------------------------------------
function s=sat2str(sat)
l='G'; if sat>56, l='J'; sat=sat-56; elseif sat>32, l='R'; sat=sat-32; end
s=sprintf('%s%02d',l,sat);
