function [x,P,s]=antphas(t0,obs,iobs,nav,inav,ionp,opt)
% antphas.m :
% estimate antenna parameters with carrier-phase observations.
%
%            Copyright(c) 2007-2011 by T.Takasu, all rights reserved
%
% version: $Revision: 1.3 $ $Date: 2008/07/21 06:45:05 $
% history: 2008/05/23 0.1 new
%          2008/07/21 0.2 fix bug on checking LLI for slip detection
%          2009/01/07 0.3 fix bug on error-stop in case of duplicated obs data
%          2011/09/13 0.4 support GLONASS, QZSS and L5

n=length(sphfun([0 0],opt.nmax)); nsat=32+24+3;
x=repmat(nan,3+nsat+n,1); P=zeros(length(x)); s=[]; nout=0; out=zeros(nsat,1);
[time,i]=unique(round(iobs(:,1))); i=[1;i+1];
for j=1:length(time)
    k=i(j):i(j+1)-1;
    [x,P,out]=udstate(x,P,obs(k,:),iobs(k,:),nav,inav,ionp,out,opt);
    [v,H,R,w]=residuals(time(j),obs(k,:),iobs(k,:),nav,inav,x,opt);
    if isempty(v), continue; end
    K=P*H'/(H*P*H'+R); xp=x+K*v; Pp=P-K*H*P;
    [v,H,R,w]=residuals(time(j),obs(k,:),iobs(k,:),nav,inav,xp,opt);
    rms=sqrt(v'*v/length(v));
    if rms>10*opt.sig
        disp(sprintf('TIME=%6.0f : NS=%2d RMS=%6.3fm *',time(j),length(v),rms));
        nout=nout+1; if nout>10, x(3+(1:nsat))=nan; nout=0; end
        continue;
    end
    x=xp; P=Pp; s=[s;w]; nout=0;
    if mod(time(j(1))+0.01,300)>0.02, continue; end
    disp(sprintf('TIME=%6.0f : NS=%2d RMS=%6.3fm',time(j),length(v),rms));
end

% double difference residuals -------------------------------------------------
function [v,H,R,w]=residuals(t,obs,iobs,nav,inav,x,opt)
[rs,dts]=ephpos(obs,iobs,nav,inav); [pos,E]=togeod(x(1:3));
n=0; nsat=32+24+3;
for sat=unique(iobs(:,2))'
    i=find(iobs(:,2)==sat&iobs(:,3)==1); if isempty(i), continue; else i=i(1); end
    j=find(iobs(:,2)==sat&iobs(:,3)==2); if isempty(j), continue; else j=j(1); end
    if any(isnan(obs([i,j],opt.f)))|any(obs([i,j],opt.f+6)<opt.minsn), continue; end
    [ri,ei]=georng(rs(:,i),x(1:3));
    [rj,ej]=georng(rs(:,j),opt.rr);
    a=E*ei; azel=[atan2(a(1),a(2)),asin(a(3))];
    if isnan(ri)|isnan(rj)|azel(2)<opt.minel, continue; end
    c=sphfun(azel+[-opt.aza,0],opt.nmax); n=n+1;
    v(n,1)=(obs(i,opt.f)-obs(j,opt.f))-(ri-rj+x(3+sat)+c*x(3+nsat+1:end));
    H(n,:)=zeros(1,length(x)); H(n,[1:3,3+sat,3+nsat+1:end])=[-ei' 1 c];
    R(n,1)=2*opt.sig^2; w(n,:)=[t,sat,azel];
end
if n<2, v=[]; H=[]; R=[]; w=[]; return; end
w=[w,v-mean(v)]; D=[ones(n-1,1),-eye(n-1)]; v=D*v; H=D*H; R=D*diag(R)*D';

% update states ----------------------------------------------------------------
function [x,P,out]=udstate(x,P,obs,iobs,nav,inav,ionp,out,opt)
nsat=32+24+3;
if ~isempty(opt.ru)
    [x,P]=setstate(x,P,1:3,opt.ru,1E-4); % constraint on position
elseif isnan(x(1))
    i=find(iobs(:,3)==1);
    ru=pntpos(obs(i,:),iobs(i,:),nav,inav,ionp,opt.minel);
    [x,P]=setstate(x,P,1:3,ru,10);
end
out=out+1; b=repmat(nan,nsat,1); 
for sat=unique(iobs(:,2))'
    i=find(iobs(:,2)==sat&iobs(:,3)==1); if isempty(i), continue; else i=i(1); end
    j=find(iobs(:,2)==sat&iobs(:,3)==2); if isempty(j), continue; else j=j(1); end
    out(sat)=0;
    b(sat)=(obs(i,opt.f)-obs(j,opt.f))-(obs(i,opt.f+3)-obs(j,opt.f+3));
    if bitand(iobs(i,opt.f+3),3)|bitand(iobs(j,opt.f+3),3)
        x(3+sat)=nan; % cycle slip by LLI
        disp(sprintf('TIME=%6.0f : SLIP %s',iobs(1,1),sat2str(sat)));
    end
end
for i=find(out>=opt.outc)'
    x(3+i)=nan; % outage count overflow
end
db=b-x(3+(1:nsat)); i=find(~isnan(db));
if length(i)>3
    if abs(mean(db(i)))>1000&std(db(i))<10
        disp(sprintf('TIME=%6.0f : RCV CLK JUMP DT=%.1fm',iobs(1,1),mean(db(i))));
        x(3+(1:nsat))=x(3+(1:nsat))+mean(db(i)); % phase-code coherency
    else
        for j=find(abs(db(i))>20)'
            x(3+i(j))=nan; % cycle slip by bias jump
            disp(sprintf('TIME=%6.0f : SLIP %s (JBS)',iobs(1,1),i(j)));
        end
    end
end
for i=find(~isnan(b)&isnan(x(3+(1:nsat))))' % reset bias
    disp(sprintf('TIME=%6.0f : RESET BIAS %s',iobs(1,1),sat2str(i)));
    [x,P]=setstate(x,P,3+i,b(i),10);
end
if isnan(x(3+nsat+1))
    n=length(x)-(3+nsat); i=3+nsat+(1:n);
    if ~isempty(opt.pcv)&n==length(pcv), x(i)=pcv; P(i,i)=pcvv;
    else [x,P]=setstate(x,P,i,zeros(1,n),1); end
end

% set states ------------------------------------------------------------------
function [x,P]=setstate(x,P,i,xi,sig)
x(i)=xi; P(i,:)=0; P(:,i)=0; P(i,i)=eye(length(i))*sig^2;

% sat to string ----------------------------------------------------------------
function s=sat2str(sat)
l='G'; if sat>56, l='J'; sat=sat-56; elseif sat>32, l='R'; sat=sat-32; end
s=sprintf('%s%02d',l,sat);
