function [rr,t,Q]=pntpos(obs,iobs,nav,inav,ionp,minel)
% pntpos.m :
% compute user position by single point positioning.
%
%            Copyright(c) 2007-2011 by T.Takasu, all rights reserved
%
% version: $Revision: 1.3 $ $Date: 2008/06/03 12:53:11 $
% history: 2008/06/02 1.0 new
%          2011/09/13 1.1 support GLONASS, QZSS

if nargin<6, minel=15*pi/180; end

C=299792458; rr=[nan;nan;nan]; t=nan; Q=zeros(4); x=zeros(4,1);
[rs,dts]=ephpos(obs,iobs,nav,inav);
for i=1:10
    v=[]; H=[]; n=0; [pos,E]=togeod(x(1:3));
    for j=1:size(rs,2)
        [r,e]=georng(rs(:,j),x(1:3));
        a=E*e; azel=[atan2(a(1),a(2)),asin(a(3))];
        if iobs(j,2)>32, continue; end % use only GPS
        if isnan(r)|isnan(obs(j,4))|azel(2)<minel, continue; end
        h=r+x(4)-C*dts(j)+ionos(iobs(1,1),pos,azel,ionp)+tropos(pos,azel);
        v=[v;obs(j,4)-h]; H=[H;-e' 1]; n=n+1; 
    end
    if n>=4, xp=x+H\v; else return; end
    if norm(xp-x)>0.1, x=xp; else break; end
end
if i<10&(n<=4|v'*v/(n-4)<10^2), rr=xp(1:3); t=iobs(1,1)-xp(4)/C; Q=inv(H'*H); end

% ionospheric model ------------------------------------------------------------
function d=ionos(t,pos,azel,ionp)
if azel(2)<=0|pos(3)<-10000|10000<pos(3), d=0; return; end
C=299792458; f=1+16*(0.53-azel(2)/pi)^3;
psi=0.0137/(azel(2)/pi+0.11)-0.022;
phi=min(max(pos(1)/pi+psi*cos(azel(1)),-0.416),0.416);
lam=pos(2)/pi+psi*sin(azel(1))/cos(phi*pi);
p=phi+0.064*cos((lam-1.617)*pi);
a=max(ionp*[1;p;p^2;p^3],[0;72000]);
x=2*pi*(mod(43200*lam+t,86400)-50400)/a(2);
if abs(x)<1.57, d=C*f*(5E-9+a(1)*(1-x^2/2+x^4/24)); else d=C*f*5E-9; end

% tropospheric model -----------------------------------------------------------
function d=tropos(pos,azel)
if azel(2)<=0|pos(3)<-10000|10000<pos(3), d=0; return; end
z=pi/2-azel(2); p=1013.25*(1-2.2557E-5*pos(3))^5.2568;
T=15.0-6.5E-3*pos(3)+273.15;
e=6.108*exp((17.15*T-4684)/(T-38.45))*0.5;
d=0.002277/cos(z)*(p+(1255/T+0.05)*e-tan(z)^2);
